% Function: contruct the convex function landscape within random points
%
% Input arguments: 
% dimension: the dimension of the original convex hull
% points_number: the number of points for initialization
% input: the dimension of the input space is (dimension-1)
% lower_bound: the setting of the lower bound for the initial points
% upper_bound: the setting of the upper bound for the initial points
% plot_flag: when dimension = 2,3, the convex function landscape can be
% visualized when plot_flag = 1
%
% Output arguments:
% fitness: the fitness value for the given input
% landscape_structure: including the coordinates for the vertexes and
% the vertex indexes for each polygon
%
% An example to apply this function：
% [fitness,landscape_structure] = convex_function_generator(3, 15, [1.2;1.5], -5, 5, 1);


function [fitness,landscape_structure] = convex_function_generator(dimension, points_number, input, lower_bound, upper_bound, plot_flag)
    if nargin < 1
        dimension = 3;
        points_number = 15;
        lower_bound = -5;
        upper_bound = 5;
        input = lower_bound*rand(dimension-1,1)*(upper_bound-lower_bound);
        plot_flag = 1;
    end
    if size(input,1) ~= (dimension-1)
       disp(['Please double check the input value.']);
    end
    %------- initialize the landscape --------------------------%
    problem = initialize_plane(lower_bound,upper_bound,dimension,points_number,plot_flag);
    landscape_structure.points = problem.rand_P;
    landscape_structure.plane_idx = problem.edge_plane.orig_plane;
    %------- find the fitness value of the given input -----------%
    fitness= fit_eval_plane(problem,input);
    disp('The input is: ');
    disp(input);
    disp(['The fitness value is ', num2str(fitness)]);   
end