% Function: Output the data from the fmincon
% Reference of plotting the output funciton is:
% Daniel Frisch (2022). outputFcn_global (https://www.mathworks.com/matlabcentral/fileexchange/80831-outputfcn_global), MATLAB Central File Exchange. Retrieved October 19, 2022.
function stop = outputFcn_global(x,optimValues,state)

% It is not supposed for live updates during optimization but for 
% later inspection, which is much more efficient. 
%
% Usage 
%   options = optimoptions( ... , 'OutputFcn',@outputFcn_global ); 
%   [XOpt,fval,exitflag,output] = fminunc(@fun, X0, options); 
%   outputFcn_global_data(k).x 
%
% See also the supplied example file. 
%
% Last Changes
%   Daniel Frisch, ISAS, 10.2020: created example
%   Daniel Frisch, ISAS, 11.2019: improved documentation 
% Created
%   Daniel Frisch, ISAS, 10.2019 
%
stop = false;
global outputFcn_global_data
switch state
  case 'init'
    outputFcn_global_data = struct();
    outputFcn_global_data.x = x;
    outputFcn_global_data.optimValues = optimValues;
    outputFcn_global_data.timerVal = tic;
  case 'iter'
    ind = length(outputFcn_global_data)+1;
    outputFcn_global_data(ind).x = x;
    outputFcn_global_data(ind).optimValues = optimValues;
    outputFcn_global_data(ind).timerVal = toc(outputFcn_global_data(1).timerVal);
  case 'done'
  otherwise
    error('wrong switch')
end
end



